@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, cast, overload
+from typing import TYPE_CHECKING, Any, Callable, Literal, cast, overload
 
 import pyarrow as pa
 import pyarrow.compute as pc
@@ -11,35 +11,38 @@ from narwhals._arrow.series_list import ArrowSeriesListNamespace
 from narwhals._arrow.series_str import ArrowSeriesStringNamespace
 from narwhals._arrow.series_struct import ArrowSeriesStructNamespace
 from narwhals._arrow.utils import (
+    arange,
     cast_for_truediv,
     chunked_array,
     extract_native,
     floordiv_compat,
+    is_array_or_scalar,
     lit,
     narwhals_to_native_dtype,
     native_to_narwhals_dtype,
     nulls_like,
     pad_series,
+    zeros,
 )
-from narwhals._compliant import EagerSeries
-from narwhals._expression_parsing import ExprKind
+from narwhals._compliant import EagerSeries, EagerSeriesHist
 from narwhals._typing_compat import assert_never
 from narwhals._utils import (
     Implementation,
     generate_temporary_column_name,
     is_list_of,
+    no_default,
     not_implemented,
 )
 from narwhals.dependencies import is_numpy_array_1d
 from narwhals.exceptions import InvalidOperationError, ShapeError
 
 if TYPE_CHECKING:
-    from collections.abc import Iterable, Iterator, Mapping, Sequence
+    from collections.abc import Iterable, Iterator, Sequence
     from types import ModuleType
 
     import pandas as pd
     import polars as pl
-    from typing_extensions import Self, TypeIs
+    from typing_extensions import Self, TypeAlias, TypeIs
 
     from narwhals._arrow.dataframe import ArrowDataFrame
     from narwhals._arrow.namespace import ArrowNamespace
@@ -51,10 +54,13 @@ if TYPE_CHECKING:
         Incomplete,
         NullPlacement,
         Order,
+        ScalarAny,
         TieBreaker,
         _AsPyType,
         _BasicDataType,
     )
+    from narwhals._compliant.series import HistData
+    from narwhals._typing import NoDefault
     from narwhals._utils import Version, _LimitedContext
     from narwhals.dtypes import DType
     from narwhals.typing import (
@@ -62,18 +68,21 @@ if TYPE_CHECKING:
         FillNullStrategy,
         Into1DArray,
         IntoDType,
+        ModeKeepStrategy,
         NonNestedLiteral,
-        NumericLiteral,
         PythonLiteral,
         RankMethod,
         RollingInterpolationMethod,
         SizedMultiIndexSelector,
-        TemporalLiteral,
         _1DArray,
         _2DArray,
         _SliceIndex,
     )
 
+    ArrowHistData: TypeAlias = (
+        "HistData[ChunkedArrayAny, list[ScalarAny] | pa.Int64Array | list[float]]"
+    )
+
 
 # TODO @dangotbanned: move into `_arrow.utils`
 # Lots of modules are importing inline
@@ -139,6 +148,16 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
             result._broadcast = self._broadcast
         return result
 
+    def _with_binary(self, op: Callable[..., ArrayOrScalar], other: Any) -> Self:
+        ser, other_native = extract_native(self, other)
+        preserve_broadcast = self._broadcast and getattr(other, "_broadcast", True)
+        return self._with_native(
+            op(ser, other_native), preserve_broadcast=preserve_broadcast
+        ).alias(self.name)
+
+    def _with_binary_right(self, op: Callable[..., ArrayOrScalar], other: Any) -> Self:
+        return self._with_binary(lambda x, y: op(y, x), other).alias(self.name)
+
     @classmethod
     def from_iterable(
         cls,
@@ -149,10 +168,15 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         dtype: IntoDType | None = None,
     ) -> Self:
         version = context._version
-        dtype_pa = narwhals_to_native_dtype(dtype, version) if dtype else None
-        return cls.from_native(
-            chunked_array([data], dtype_pa), name=name, context=context
-        )
+        if dtype is not None:
+            dtype_pa: pa.DataType | None = narwhals_to_native_dtype(dtype, version)
+            if is_array_or_scalar(data):
+                data = data.cast(dtype_pa)
+                dtype_pa = None
+            native = data if cls._is_native(data) else chunked_array([data], dtype_pa)
+        else:
+            native = chunked_array([data])
+        return cls.from_native(native, context=context, name=name)
 
     def _from_scalar(self, value: Any) -> Self:
         if hasattr(value, "as_py"):
@@ -178,16 +202,18 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
     @classmethod
     def _align_full_broadcast(cls, *series: Self) -> Sequence[Self]:
         lengths = [len(s) for s in series]
-        max_length = max(lengths)
-        fast_path = all(_len == max_length for _len in lengths)
+        target_length = max(
+            length for length, s in zip(lengths, series) if not s._broadcast
+        )
+        fast_path = all(_len == target_length for _len in lengths)
         if fast_path:
             return series
         reshaped = []
         for s in series:
             if s._broadcast:
-                compliant = s._with_native(pa.repeat(s.native[0], max_length))
-            elif (actual_len := len(s)) != max_length:
-                msg = f"Expected object of length {max_length}, got {actual_len}."
+                compliant = s._with_native(pa.repeat(s.native[0], target_length))
+            elif (actual_len := len(s)) != target_length:
+                msg = f"Expected object of length {target_length}, got {actual_len}."
                 raise ShapeError(msg)
             else:
                 compliant = s
@@ -200,106 +226,89 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         return ArrowNamespace(version=self._version)
 
     def __eq__(self, other: object) -> Self:  # type: ignore[override]
-        other = cast("PythonLiteral | ArrowSeries | None", other)
-        ser, rhs = extract_native(self, other)
-        return self._with_native(pc.equal(ser, rhs))
+        return self._with_binary(pc.equal, other)
 
     def __ne__(self, other: object) -> Self:  # type: ignore[override]
-        other = cast("PythonLiteral | ArrowSeries | None", other)
-        ser, rhs = extract_native(self, other)
-        return self._with_native(pc.not_equal(ser, rhs))
+        return self._with_binary(pc.not_equal, other)
 
     def __ge__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.greater_equal(ser, other))
+        return self._with_binary(pc.greater_equal, other)
 
     def __gt__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.greater(ser, other))
+        return self._with_binary(pc.greater, other)
 
     def __le__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.less_equal(ser, other))
+        return self._with_binary(pc.less_equal, other)
 
     def __lt__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.less(ser, other))
+        return self._with_binary(pc.less, other)
 
     def __and__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.and_kleene(ser, other))  # type: ignore[arg-type]
+        return self._with_binary(pc.and_kleene, other)
 
     def __rand__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.and_kleene(other, ser))  # type: ignore[arg-type]
+        return self._with_binary_right(pc.and_kleene, other)
 
     def __or__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.or_kleene(ser, other))  # type: ignore[arg-type]
+        return self._with_binary_right(pc.or_kleene, other)
 
     def __ror__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.or_kleene(other, ser))  # type: ignore[arg-type]
+        return self._with_binary_right(pc.or_kleene, other)
 
     def __add__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.add(ser, other))
+        return self._with_binary(pc.add, other)
 
     def __radd__(self, other: Any) -> Self:
-        return self + other
+        return self._with_binary_right(pc.add, other)
 
     def __sub__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.subtract(ser, other))
+        return self._with_binary(pc.subtract, other)
 
     def __rsub__(self, other: Any) -> Self:
-        return (self - other) * (-1)
+        return self._with_binary_right(pc.subtract, other)
 
     def __mul__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.multiply(ser, other))
+        return self._with_binary(pc.multiply, other)
 
     def __rmul__(self, other: Any) -> Self:
-        return self * other
+        return self._with_binary_right(pc.multiply, other)
 
     def __pow__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.power(ser, other))
+        return self._with_binary(pc.power, other)
 
     def __rpow__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.power(other, ser))
+        return self._with_binary_right(pc.power, other)
 
     def __floordiv__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(floordiv_compat(ser, other))
+        return self._with_binary(floordiv_compat, other)
 
     def __rfloordiv__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(floordiv_compat(other, ser))
+        return self._with_binary_right(floordiv_compat, other)
 
     def __truediv__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.divide(*cast_for_truediv(ser, other)))  # type: ignore[type-var]
+        return self._with_binary(lambda x, y: pc.divide(*cast_for_truediv(x, y)), other)
 
     def __rtruediv__(self, other: Any) -> Self:
-        ser, other = extract_native(self, other)
-        return self._with_native(pc.divide(*cast_for_truediv(other, ser)))  # type: ignore[type-var]
+        return self._with_binary_right(
+            lambda x, y: pc.divide(*cast_for_truediv(x, y)), other
+        )
 
     def __mod__(self, other: Any) -> Self:
+        preserve_broadcast = self._broadcast and getattr(other, "_broadcast", True)
         floor_div = (self // other).native
         ser, other = extract_native(self, other)
         res = pc.subtract(ser, pc.multiply(floor_div, other))
-        return self._with_native(res)
+        return self._with_native(res, preserve_broadcast=preserve_broadcast)
 
     def __rmod__(self, other: Any) -> Self:
+        preserve_broadcast = self._broadcast and getattr(other, "_broadcast", True)
         floor_div = (other // self).native
         ser, other = extract_native(self, other)
         res = pc.subtract(other, pc.multiply(floor_div, ser))
-        return self._with_native(res)
+        return self._with_native(res, preserve_broadcast=preserve_broadcast)
 
     def __invert__(self) -> Self:
-        return self._with_native(pc.invert(self.native))
+        return self._with_native(pc.invert(self.native), preserve_broadcast=True)
 
     @property
     def _type(self) -> pa.DataType:
@@ -316,12 +325,19 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
             other_native = predicate
         return self._with_native(self.native.filter(other_native))
 
+    def first(self, *, _return_py_scalar: bool = True) -> PythonLiteral:
+        result = self.native[0] if len(self.native) else None
+        return maybe_extract_py_scalar(result, _return_py_scalar)
+
+    def last(self, *, _return_py_scalar: bool = True) -> PythonLiteral:
+        ca = self.native
+        result = ca[height - 1] if (height := len(ca)) else None
+        return maybe_extract_py_scalar(result, _return_py_scalar)
+
     def mean(self, *, _return_py_scalar: bool = True) -> float:
         return maybe_extract_py_scalar(pc.mean(self.native), _return_py_scalar)
 
     def median(self, *, _return_py_scalar: bool = True) -> float:
-        from narwhals.exceptions import InvalidOperationError
-
         if not self.dtype.is_numeric():
             msg = "`median` operation not supported for non-numeric input type."
             raise InvalidOperationError(msg)
@@ -361,12 +377,12 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
             return self._with_native(self.native)
         return self._with_native(pa.concat_arrays(arrays))
 
-    def std(self, ddof: int, *, _return_py_scalar: bool = True) -> float:
+    def std(self, *, ddof: int, _return_py_scalar: bool = True) -> float:
         return maybe_extract_py_scalar(
             pc.stddev(self.native, ddof=ddof), _return_py_scalar
         )
 
-    def var(self, ddof: int, *, _return_py_scalar: bool = True) -> float:
+    def var(self, *, ddof: int, _return_py_scalar: bool = True) -> float:
         return maybe_extract_py_scalar(
             pc.variance(self.native, ddof=ddof), _return_py_scalar
         )
@@ -375,29 +391,27 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         ser_not_null = self.native.drop_null()
         if len(ser_not_null) == 0:
             return None
-        elif len(ser_not_null) == 1:
+        if len(ser_not_null) == 1:
             return float("nan")
-        elif len(ser_not_null) == 2:
+        if len(ser_not_null) == 2:
             return 0.0
-        else:
-            m = pc.subtract(ser_not_null, pc.mean(ser_not_null))
-            m2 = pc.mean(pc.power(m, lit(2)))
-            m3 = pc.mean(pc.power(m, lit(3)))
-            biased_population_skewness = pc.divide(m3, pc.power(m2, lit(1.5)))
-            return maybe_extract_py_scalar(biased_population_skewness, _return_py_scalar)
+        m = pc.subtract(ser_not_null, pc.mean(ser_not_null))
+        m2 = pc.mean(pc.power(m, lit(2)))
+        m3 = pc.mean(pc.power(m, lit(3)))
+        biased_population_skewness = pc.divide(m3, pc.power(m2, lit(1.5)))
+        return maybe_extract_py_scalar(biased_population_skewness, _return_py_scalar)
 
     def kurtosis(self, *, _return_py_scalar: bool = True) -> float | None:
         ser_not_null = self.native.drop_null()
         if len(ser_not_null) == 0:
             return None
-        elif len(ser_not_null) == 1:
+        if len(ser_not_null) == 1:
             return float("nan")
-        else:
-            m = pc.subtract(ser_not_null, pc.mean(ser_not_null))
-            m2 = pc.mean(pc.power(m, lit(2)))
-            m4 = pc.mean(pc.power(m, lit(4)))
-            k = pc.subtract(pc.divide(m4, pc.power(m2, lit(2))), lit(3))
-            return maybe_extract_py_scalar(k, _return_py_scalar)
+        m = pc.subtract(ser_not_null, pc.mean(ser_not_null))
+        m2 = pc.mean(pc.power(m, lit(2)))
+        m4 = pc.mean(pc.power(m, lit(4)))
+        k = pc.subtract(pc.divide(m4, pc.power(m2, lit(2))), lit(3))
+        return maybe_extract_py_scalar(k, _return_py_scalar)
 
     def count(self, *, _return_py_scalar: bool = True) -> int:
         return maybe_extract_py_scalar(pc.count(self.native), _return_py_scalar)
@@ -479,9 +493,9 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         return self.native.to_numpy()
 
     def alias(self, name: str) -> Self:
-        result = self.__class__(self.native, name=name, version=self._version)
-        result._broadcast = self._broadcast
-        return result
+        ret = self.__class__(self.native, name=name, version=self._version)
+        ret._broadcast = self._broadcast
+        return ret
 
     @property
     def dtype(self) -> DType:
@@ -504,6 +518,12 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
             pc.round(self.native, decimals, round_mode="half_towards_infinity")
         )
 
+    def floor(self) -> Self:
+        return self._with_native(pc.floor(self.native))
+
+    def ceil(self) -> Self:
+        return self._with_native(pc.ceil(self.native))
+
     def diff(self) -> Self:
         return self._with_native(pc.pairwise_diff(self.native.combine_chunks()))
 
@@ -558,16 +578,14 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
     def head(self, n: int) -> Self:
         if n >= 0:
             return self._with_native(self.native.slice(0, n))
-        else:
-            num_rows = len(self)
-            return self._with_native(self.native.slice(0, max(0, num_rows + n)))
+        num_rows = len(self)
+        return self._with_native(self.native.slice(0, max(0, num_rows + n)))
 
     def tail(self, n: int) -> Self:
         if n >= 0:
             num_rows = len(self)
             return self._with_native(self.native.slice(max(0, num_rows - n)))
-        else:
-            return self._with_native(self.native.slice(abs(n)))
+        return self._with_native(self.native.slice(abs(n)))
 
     def is_in(self, other: Any) -> Self:
         if self._is_native(other):
@@ -643,46 +661,49 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         mask = rng.choice(idx, size=n, replace=with_replacement)
         return self._with_native(self.native.take(mask))
 
+    def fill_nan(self, value: float | None) -> Self:
+        result = pc.if_else(pc.is_nan(self.native), value, self.native)
+        return self._with_native(result, preserve_broadcast=True)
+
     def fill_null(
         self,
         value: Self | NonNestedLiteral,
         strategy: FillNullStrategy | None,
         limit: int | None,
     ) -> Self:
-        import numpy as np  # ignore-banned-import
-
-        def fill_aux(
-            arr: ChunkedArrayAny, limit: int, direction: FillNullStrategy | None
-        ) -> ArrayAny:
+        def fill_null_forward_limit(arr: ChunkedArrayAny, limit: int) -> ArrayAny:
             # this algorithm first finds the indices of the valid values to fill all the null value positions
             # then it calculates the distance of each new index and the original index
             # if the distance is equal to or less than the limit and the original value is null, it is replaced
-            valid_mask = pc.is_valid(arr)
-            indices = pa.array(np.arange(len(arr)), type=pa.int64())
-            if direction == "forward":
-                valid_index = np.maximum.accumulate(np.where(valid_mask, indices, -1))
-                distance = indices - valid_index
-            else:
-                valid_index = np.minimum.accumulate(
-                    np.where(valid_mask[::-1], indices[::-1], len(arr))
-                )[::-1]
-                distance = valid_index - indices
+            sentinel = lit(-1)
+            is_not_null = pc.is_valid(arr)
+            index = arange(0, len(arr), 1)
+            index_not_null = pc.cumulative_max(pc.if_else(is_not_null, index, sentinel))
+            # NOTE: The correction here is for nulls at either end of the array
+            # They should be preserved when the `strategy` would need an out-of-bounds index
+            not_oob = pc.not_equal(index_not_null, sentinel)
+            index_not_null = pc.if_else(not_oob, index_not_null, None)
+            beyond_limit = pc.greater(pc.subtract(index, index_not_null), lit(limit))
             return pc.if_else(
-                pc.and_(pc.is_null(arr), pc.less_equal(distance, lit(limit))),  # pyright: ignore[reportArgumentType, reportCallIssue]
-                arr.take(valid_index),
-                arr,
+                pc.or_(is_not_null, beyond_limit), arr, arr.take(index_not_null)
             )
 
+        native = self.native
         if value is not None:
             _, native_value = extract_native(self, value)
-            series: ArrayOrScalar = pc.fill_null(self.native, native_value)
+            series: ArrayOrScalar = pc.fill_null(native, native_value)
         elif limit is None:
-            fill_func = (
-                pc.fill_null_forward if strategy == "forward" else pc.fill_null_backward
+            series = (
+                pc.fill_null_forward(native)
+                if strategy == "forward"
+                else pc.fill_null_backward(native)
             )
-            series = fill_func(self.native)
         else:
-            series = fill_aux(self.native, limit, strategy)
+            series = (
+                fill_null_forward_limit(native, limit)
+                if strategy == "forward"
+                else fill_null_forward_limit(native[::-1], limit)[::-1]
+            )
         return self._with_native(series, preserve_broadcast=True)
 
     def to_frame(self) -> ArrowDataFrame:
@@ -744,30 +765,50 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
             result = pc.all(pc.less_equal(self.native[:-1], self.native[1:]))
         return maybe_extract_py_scalar(result, return_py_scalar=True)
 
-    def unique(self, *, maintain_order: bool) -> Self:
+    def unique(self, *, maintain_order: bool = True) -> Self:
         # TODO(marco): `pc.unique` seems to always maintain order, is that guaranteed?
         return self._with_native(self.native.unique())
 
     def replace_strict(
         self,
-        old: Sequence[Any] | Mapping[Any, Any],
+        default: Any | NoDefault,
+        old: Sequence[Any],
         new: Sequence[Any],
         *,
         return_dtype: IntoDType | None,
     ) -> Self:
         # https://stackoverflow.com/a/79111029/4451315
         idxs = pc.index_in(self.native, pa.array(old))
+        # Tracks which values were found in the mapping (even if mapped to None)
+        was_matched = pc.is_valid(idxs)
         result_native = pc.take(pa.array(new), idxs)
         if return_dtype is not None:
-            result_native.cast(narwhals_to_native_dtype(return_dtype, self._version))
-        result = self._with_native(result_native)
-        if result.is_null().sum() != self.is_null().sum():
-            msg = (
-                "replace_strict did not replace all non-null values.\n\n"
-                "The following did not get replaced: "
-                f"{self.filter(~self.is_null() & result.is_null()).unique(maintain_order=False).to_list()}"
+            result_native = result_native.cast(
+                narwhals_to_native_dtype(return_dtype, self._version)
             )
-            raise ValueError(msg)
+        result = self._with_native(result_native)
+        if default is no_default:
+            # Check that all non-null input values were matched
+            # (result may have more nulls if mapping contains {value: None})
+            unmatched_mask = pc.and_(pc.is_valid(self.native), pc.invert(was_matched))
+            if maybe_extract_py_scalar(
+                pc.any(unmatched_mask, min_count=0), return_py_scalar=True
+            ):
+                unmatched_values = (
+                    self.filter(self._with_native(unmatched_mask))
+                    .unique(maintain_order=False)
+                    .to_list()
+                )
+                msg = (
+                    "replace_strict did not replace all non-null values.\n\n"
+                    f"The following did not get replaced: {unmatched_values}"
+                )
+                raise InvalidOperationError(msg)
+        else:
+            result_native, default = extract_native(result, default)
+            # Only fill with default where the value wasn't matched (not where result is null due to mapping)
+            # If was_matched, keep result.native; otherwise use default
+            result = self._with_native(pc.if_else(was_matched, result.native, default))
         return result
 
     def sort(self, *, descending: bool, nulls_last: bool) -> Self:
@@ -826,35 +867,34 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
     def gather_every(self, n: int, offset: int = 0) -> Self:
         return self._with_native(self.native[offset::n])
 
-    def clip(
-        self,
-        lower_bound: Self | NumericLiteral | TemporalLiteral | None,
-        upper_bound: Self | NumericLiteral | TemporalLiteral | None,
-    ) -> Self:
-        _, lower = extract_native(self, lower_bound) if lower_bound else (None, None)
-        _, upper = extract_native(self, upper_bound) if upper_bound else (None, None)
-
-        if lower is None:
-            return self._with_native(pc.min_element_wise(self.native, upper))
-        if upper is None:
-            return self._with_native(pc.max_element_wise(self.native, lower))
+    def clip(self, lower_bound: Self, upper_bound: Self) -> Self:
+        _, lower = extract_native(self, lower_bound)
+        _, upper = extract_native(self, upper_bound)
         return self._with_native(
             pc.max_element_wise(pc.min_element_wise(self.native, upper), lower)
         )
 
+    def clip_lower(self, lower_bound: Self) -> Self:
+        _, lower = extract_native(self, lower_bound)
+        return self._with_native(pc.max_element_wise(self.native, lower))
+
+    def clip_upper(self, upper_bound: Self) -> Self:
+        _, upper = extract_native(self, upper_bound)
+        return self._with_native(pc.min_element_wise(self.native, upper))
+
     def to_arrow(self) -> ArrayAny:
         return self.native.combine_chunks()
 
-    def mode(self) -> ArrowSeries:
+    def mode(self, *, keep: ModeKeepStrategy) -> ArrowSeries:
         plx = self.__narwhals_namespace__()
         col_token = generate_temporary_column_name(n_bytes=8, columns=[self.name])
         counts = self.value_counts(
             name=col_token, normalize=False, sort=False, parallel=False
         )
-        return counts.filter(
-            plx.col(col_token)
-            == plx.col(col_token).max().broadcast(kind=ExprKind.AGGREGATION)
+        result = counts.filter(
+            plx.col(col_token) == plx.col(col_token).max().broadcast()
         ).get_column(self.name)
+        return result.head(1) if keep == "any" else result
 
     def is_finite(self) -> Self:
         return self._with_native(pc.is_finite(self.native))
@@ -1017,101 +1057,22 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         result = pc.if_else(null_mask, lit(None, rank.type), rank)
         return self._with_native(result)
 
-    def hist(  # noqa: C901, PLR0912, PLR0915
-        self,
-        bins: list[float | int] | None,
-        *,
-        bin_count: int | None,
-        include_breakpoint: bool,
+    def hist_from_bins(
+        self, bins: list[float], *, include_breakpoint: bool
     ) -> ArrowDataFrame:
-        import numpy as np  # ignore-banned-import
-
-        from narwhals._arrow.dataframe import ArrowDataFrame
-
-        def _hist_from_bin_count(bin_count: int):  # type: ignore[no-untyped-def] # noqa: ANN202
-            d = pc.min_max(self.native)
-            lower, upper = d["min"].as_py(), d["max"].as_py()
-            if lower == upper:
-                lower -= 0.5
-                upper += 0.5
-            bins = np.linspace(lower, upper, bin_count + 1)
-            return _hist_from_bins(bins)
-
-        def _hist_from_bins(bins: Sequence[int | float]):  # type: ignore[no-untyped-def] # noqa: ANN202
-            bin_indices = np.searchsorted(bins, self.native, side="left")
-            bin_indices = pc.if_else(  # lowest bin is inclusive
-                pc.equal(self.native, lit(bins[0])), 1, bin_indices
-            )
-
-            # align unique categories and counts appropriately
-            obs_cats, obs_counts = np.unique(bin_indices, return_counts=True)
-            obj_cats = np.arange(1, len(bins))
-            counts = np.zeros_like(obj_cats)
-            counts[np.isin(obj_cats, obs_cats)] = obs_counts[np.isin(obs_cats, obj_cats)]
-
-            bin_right = bins[1:]
-            return counts, bin_right
-
-        counts: Sequence[int | float | pa.Scalar[Any]] | np.typing.ArrayLike
-        bin_right: Sequence[int | float | pa.Scalar[Any]] | np.typing.ArrayLike
-
-        data_count = pc.sum(
-            pc.invert(pc.or_(pc.is_nan(self.native), pc.is_null(self.native))).cast(
-                pa.uint8()
-            ),
-            min_count=0,
+        return (
+            _ArrowHist.from_series(self, include_breakpoint=include_breakpoint)
+            .with_bins(bins)
+            .to_frame()
         )
-        if bins is not None:
-            if len(bins) < 2:
-                counts, bin_right = [], []
 
-            elif data_count == pa.scalar(0, type=pa.uint64()):  # type:ignore[comparison-overlap]
-                counts = np.zeros(len(bins) - 1)
-                bin_right = bins[1:]
-
-            elif len(bins) == 2:
-                counts = [
-                    pc.sum(
-                        pc.and_(
-                            pc.greater_equal(self.native, lit(float(bins[0]))),
-                            pc.less_equal(self.native, lit(float(bins[1]))),
-                        ).cast(pa.uint8())
-                    )
-                ]
-                bin_right = [bins[-1]]
-            else:
-                counts, bin_right = _hist_from_bins(bins)
-
-        elif bin_count is not None:
-            if bin_count == 0:
-                counts, bin_right = [], []
-            elif data_count == pa.scalar(0, type=pa.uint64()):  # type:ignore[comparison-overlap]
-                counts, bin_right = (
-                    np.zeros(bin_count),
-                    np.linspace(0, 1, bin_count + 1)[1:],
-                )
-            elif bin_count == 1:
-                d = pc.min_max(self.native)
-                lower, upper = d["min"], d["max"]
-                if lower == upper:
-                    counts, bin_right = [data_count], [pc.add(upper, pa.scalar(0.5))]
-                else:
-                    counts, bin_right = [data_count], [upper]
-            else:
-                counts, bin_right = _hist_from_bin_count(bin_count)
-
-        else:  # pragma: no cover
-            # caller guarantees that either bins or bin_count is specified
-            msg = "must provide one of `bin_count` or `bins`"
-            raise InvalidOperationError(msg)
-
-        data: dict[str, Any] = {}
-        if include_breakpoint:
-            data["breakpoint"] = bin_right
-        data["count"] = counts
-
-        return ArrowDataFrame(
-            pa.Table.from_pydict(data), version=self._version, validate_column_names=True
+    def hist_from_bin_count(
+        self, bin_count: int, *, include_breakpoint: bool
+    ) -> ArrowDataFrame:
+        return (
+            _ArrowHist.from_series(self, include_breakpoint=include_breakpoint)
+            .with_bin_count(bin_count)
+            .to_frame()
         )
 
     def __iter__(self) -> Iterator[Any]:
@@ -1131,8 +1092,6 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
                 pc.is_in(other_, self.native), return_py_scalar=True
             )
         except (ArrowInvalid, ArrowNotImplementedError, ArrowTypeError) as exc:
-            from narwhals.exceptions import InvalidOperationError
-
             msg = f"Unable to compare other of type {type(other)} with series of type {self.dtype}."
             raise InvalidOperationError(msg) from exc
 
@@ -1145,6 +1104,11 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
     def sqrt(self) -> Self:
         return self._with_native(pc.sqrt(self.native))
 
+    def any_value(
+        self, *, ignore_nulls: bool, _return_py_scalar: bool = True
+    ) -> PythonLiteral:
+        return self.drop_nulls().first() if ignore_nulls else self.first()
+
     @property
     def dt(self) -> ArrowSeriesDateTimeNamespace:
         return ArrowSeriesDateTimeNamespace(self)
@@ -1166,3 +1130,90 @@ class ArrowSeries(EagerSeries["ChunkedArrayAny"]):
         return ArrowSeriesStructNamespace(self)
 
     ewm_mean = not_implemented()
+
+
+class _ArrowHist(
+    EagerSeriesHist["ChunkedArrayAny", "list[ScalarAny] | pa.Int64Array | list[float]"]
+):
+    _series: ArrowSeries
+
+    def to_frame(self) -> ArrowDataFrame:
+        # NOTE: Constructor typing is too strict for `TypedDict`
+        table: Incomplete = pa.Table.from_pydict
+        from_native = self._series.__narwhals_namespace__()._dataframe.from_native
+        return from_native(table(self._data), context=self._series)
+
+    # NOTE: *Could* be handled at narwhals-level
+    def is_empty_series(self) -> bool:
+        # NOTE: `ChunkedArray.combine_chunks` returns the concrete array type
+        # Stubs say `Array[pa.BooleanScalar]`, which is missing properties
+        # https://github.com/zen-xu/pyarrow-stubs/blob/6bedee748bc74feb8513b24bf43d64b24c7fddc8/pyarrow-stubs/__lib_pxi/array.pyi#L2395-L2399
+        is_null = self.native.is_null(nan_is_null=True)
+        arr = cast("pa.BooleanArray", is_null.combine_chunks())
+        return arr.false_count == 0
+
+    # NOTE: *Could* be handled at narwhals-level, **iff** we add `nw.repeat`, `nw.linear_space`
+    # See https://github.com/narwhals-dev/narwhals/pull/2839#discussion_r2215630696
+    def series_empty(self, arg: int | list[float], /) -> ArrowHistData:
+        count = self._zeros(arg)
+        if self._breakpoint:
+            return {"breakpoint": self._calculate_breakpoint(arg), "count": count}
+        return {"count": count}
+
+    def _zeros(self, arg: int | list[float], /) -> pa.Int64Array:
+        return zeros(arg) if isinstance(arg, int) else zeros(len(arg) - 1)
+
+    def _linear_space(
+        self,
+        start: float,
+        end: float,
+        num_samples: int,
+        *,
+        closed: Literal["both", "none"] = "both",
+    ) -> _1DArray:
+        from numpy import linspace  # ignore-banned-import
+
+        return linspace(start=start, stop=end, num=num_samples, endpoint=closed == "both")
+
+    def _calculate_bins(self, bin_count: int) -> _1DArray:
+        """Prepare bins for histogram calculation from bin_count."""
+        d = pc.min_max(self.native)
+        lower, upper = d["min"].as_py(), d["max"].as_py()
+        if lower == upper:
+            lower -= 0.5
+            upper += 0.5
+        return self._linear_space(lower, upper, bin_count + 1)
+
+    def _calculate_hist(self, bins: list[float] | _1DArray) -> ArrowHistData:
+        ser = self.native
+        # NOTE: `mypy` refuses to resolve `ndarray.__getitem__`
+        # Previously annotated as `list[float]`, but
+        # - wasn't accurate to how we implemented it
+        # - `pa.scalar` overloads fail to match on `float | np.float64` (but runtime is fine)
+        bins = cast("list[float]", bins)
+        # Handle single bin case
+        if len(bins) == 2:
+            is_between_bins = pc.and_(
+                pc.greater_equal(ser, lit(bins[0])), pc.less_equal(ser, lit(bins[1]))
+            )
+            count = pc.sum(is_between_bins.cast(pa.uint8()))
+            if self._breakpoint:
+                return {"breakpoint": [bins[-1]], "count": [count]}
+            return {"count": [count]}
+
+        # Handle multiple bins
+        import numpy as np  # ignore-banned-import
+
+        bin_indices = np.searchsorted(bins, ser, side="left")
+        # lowest bin is inclusive
+        bin_indices = pc.if_else(pc.equal(ser, lit(bins[0])), 1, bin_indices)
+
+        # Align unique categories and counts appropriately
+        obs_cats, obs_counts = np.unique(bin_indices, return_counts=True)
+        obj_cats = np.arange(1, len(bins))
+        counts = np.zeros_like(obj_cats)
+        counts[np.isin(obj_cats, obs_cats)] = obs_counts[np.isin(obs_cats, obj_cats)]
+
+        if self._breakpoint:
+            return {"breakpoint": bins[1:], "count": counts}
+        return {"count": counts}
